-- NEW: modified to work with linux
-- File routines for Euphoria Pre-Processor
-- path = addSlash(path)
--  adds a \ to path if it needs one
-- path = absolutePath(path)
--  gets the absolute path of a path (\windows -> c:\windows)
-- path = absoluteName(path)
--  gets the absolute name of a file (\windows\win.ini -> c:\windows\win.ini)
-- path = getName(path)
--  gets the name of the file in path (\epp.ex -> epp.ex)
-- path = includeName(path)
--  gets the absolute name of an included file
--  (graphics.e -> c:\euphoria\include\graphics.e)
-- file_data = loadFile(path/file_number)
--  Loads a file into a sequence
include file.e      -- for use with includeName to see whether a file exists
		    -- also for current_dir()
include wildcard.e  -- to make the paths in lower case (so you can use compare)
include misc.e      -- for platform
integer pathSeperator
if platform() = LINUX then
    pathSeperator = '/'
else
    pathSeperator = '\\'
end if

global function addSlash(sequence s)
    if s[length(s)] != pathSeperator then
	return s & pathSeperator
    end if
    return s
end function
procedure cd(sequence p)
    if platform() != LINUX then
	if length(p) > 2 then
	    if p[2] = ':' then -- If it contains a drive
		system(p[1..2],2)
	    end if
	end if
    end if
    system("cd " & p,2)
end procedure
function split_dirs(sequence p)
    sequence r, temp
    r = {}
    temp = ""
    for i = 1 to length(p) do
	if p[i] = pathSeperator then
	    if length(temp) then
		r = append(r,temp)
		temp = ""
	    end if
	else
	    temp &= p[i]
	end if
    end for
    if length(temp) then
	r = append(r,temp)
    end if
    return r
end function
function absolutePath_linux(sequence p)
    sequence tree, temp
    integer i
    if p[1] = '~' then
	p = "/home/" & getenv("USER") & p[2..length(p)]
    end if
    if p[1] = '/' then
	tree = split_dirs(p)
    else
	tree = split_dirs(current_dir())
    end if
    i = 1
    while i <= length(tree) do
	if equal(tree[i],".") then
	    tree = tree[1..i-1] & tree[i+1..length(tree)]
	elsif equal(tree[i],"..") then
	    tree = tree[1..i-2] & tree[i+1..length(tree)]
	else
	    i += 1
	end if
    end while
    temp = "/"
    for j = 1 to length(tree) do
	temp &= tree[j] & '/'
    end for
    return temp
end function
global function absolutePath(sequence p)
    sequence curdir, ret
    curdir = current_dir()
    if length(p) = 0 then
	if platform() = LINUX then
	    return addSlash(curdir)
	else
	    return addSlash(lower(curdir))
	end if
    end if
    if platform() = LINUX then
	return absolutePath_linux(p)
    end if
    cd(p)
    ret = current_dir()
    cd(curdir)
    if platform() = LINUX then
	return addSlash(ret)
    else
	return addSlash(lower(ret))
    end if
end function
function getName(sequence s)
    for i = length(s) to 1 by -1 do
	if s[i] = pathSeperator then
	    return s[i+1..length(s)]
	end if
    end for
    return s
end function
global function absoluteName(sequence s)
    sequence path, file
    file = s
    path = ""
    for i = length(s) to 1 by -1 do
	if s[i] = pathSeperator then
	    path = s[1..i]
	    file = s[i+1..length(s)]
	    exit
	end if
    end for
    if platform() = LINUX then
	return absolutePath(path) & file
    else
	return absolutePath(path) & lower(file)
    end if
end function
global function includeName(sequence s)
    if sequence(dir(s)) then
	return absoluteName(s) -- Found it in current directory
    else
	s = absolutePath(getenv("EUDIR")) & "include" & pathSeperator & s
	if sequence(dir(s)) then
	    return s -- Found it in include directory
	end if
    end if
    return "" -- The file couldn't be found in the current directory
	      -- or in the include directory.
end function
global function loadFile(object x)
    sequence r
    integer t
    if sequence(x) then
	x = open(x,"r")
    end if
    if x = -1 then
	return -1
    end if
    r = {}
    t = getc(x)
    while t != -1 do
	r = r & t
	t = getc(x)
    end while
    close(x)
    return r
end function
